# Generated by Django 3.2.25 on 2024-05-30 02:27

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_details", "0006_userdetails_orcid"),
    ]

    operations = [
        migrations.CreateModel(
            name="EducationLevel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(help_text="The name of the education level", max_length=255, unique=True)),
            ],
            options={
                "ordering": ["name"],
            },
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="name",
            field=models.CharField(help_text="The name of the ethnicity", max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="gender",
            name="name",
            field=models.CharField(help_text="The name of the gender", max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="race",
            name="name",
            field=models.CharField(help_text="The name of the race", max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="userdetails",
            name="emergency_contact",
            field=models.CharField(blank=True, help_text="The user's emergency contact information", max_length=255),
        ),
        migrations.AddField(
            model_name="userdetails",
            name="education_level",
            field=models.ForeignKey(
                blank=True,
                help_text="The user's education level",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="NEMO_user_details.educationlevel",
            ),
        ),
    ]



# Generated by Django 4.2.19 on 2025-05-29 20:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_details", "0007_userdetails_education_level"),
    ]

    operations = [
        migrations.AddField(
            model_name="userdetails",
            name="google_scholar_id",
            field=models.CharField(
                blank=True,
                help_text="The user's Google Scholar ID",
                max_length=255,
                null=True,
                verbose_name="Google scholar ID",
            ),
        ),
        migrations.AddField(
            model_name="userdetails",
            name="researcher_id",
            field=models.CharField(
                blank=True,
                help_text="The user's Web of Science Researcher ID",
                max_length=255,
                null=True,
                verbose_name="Researcher ID",
            ),
        ),
        migrations.AddField(
            model_name="userdetails",
            name="scopus_id",
            field=models.CharField(
                blank=True, help_text="The user's Scopus ID", max_length=255, null=True, verbose_name="Scopus ID"
            ),
        ),
    ]



# Generated by Django 3.2.16 on 2023-01-20 14:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_details", "0002_version_1_1_0"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="userdetails",
            name="rate_category_id",
        ),
        migrations.RemoveField(
            model_name="userdetails",
            name="rate_category_type",
        ),
    ]



from NEMO.decorators import any_staff_required, replace_function
from NEMO.models import User
from NEMO.typing import QuerySetType
from NEMO.utilities import render_combine_responses
from NEMO.views import email as email_views
from django.contrib.auth.models import Group
from django.views.decorators.http import require_GET

from NEMO_user_details.customizations import UserDetailsCustomization


@any_staff_required
@require_GET
def email_broadcast(request, audience=""):
    original_response = email_views.email_broadcast(request, audience)

    if not UserDetailsCustomization.get_bool("user_details_enable_groups"):
        return original_response

    email_broadcast_dictionary = {}

    if audience == "group":
        email_broadcast_dictionary["group_types"] = Group.objects.all()

    return render_combine_responses(
        request,
        original_response,
        "NEMO_user_details/email_broadcast.html",
        email_broadcast_dictionary,
    )


@replace_function("NEMO.views.email.get_users_for_email")
def new_get_users_for_email(old_function, audience: str, selection, no_type: bool) -> (QuerySetType[User], str):
    if audience != "group":
        return old_function(audience, selection, no_type)
    else:
        user_group_list = User.objects.none()
        for and_group in selection:
            user_group = User.objects.all()
            for group_pk in and_group.split(" "):
                user_group &= User.objects.filter(groups__in=[int(group_pk)])

            user_group_list |= user_group

        return user_group_list.distinct(), None



from NEMO.admin import UserAdmin
from NEMO.models import User
from django.contrib import admin

from NEMO_user_details.forms import UserDetailsAdminForm
from NEMO_user_details.models import EducationLevel, Ethnicity, Gender, Race, UserDetails


class UserDetailsInline(admin.StackedInline):
    form = UserDetailsAdminForm
    model = UserDetails
    can_delete = False
    min_num = 1


class NewUserAdmin(UserAdmin):
    inlines = [UserDetailsInline] + list(UserAdmin.inlines)


# Re-register UserAdmin
admin.site.unregister(User)
admin.site.register(User, NewUserAdmin)

admin.site.register(Race)
admin.site.register(Gender)
admin.site.register(Ethnicity)
admin.site.register(EducationLevel)



from copy import deepcopy

from NEMO.models import User
from NEMO.serializers import ModelSerializer, UserSerializer
from NEMO.views.api import ModelViewSet, UserViewSet
from rest_framework.fields import CharField

from NEMO_user_details.customizations import UserDetailsCustomization
from NEMO_user_details.models import EducationLevel, Ethnicity, Gender, Race, UserDetails, get_user_details


class GenderSerializer(ModelSerializer):
    class Meta:
        model = Gender
        fields = "__all__"


class GenderViewSet(ModelViewSet):
    filename = "user_details_genders"
    queryset = Gender.objects.all()
    serializer_class = GenderSerializer
    filterset_fields = {
        "id": ["exact", "in", "isnull"],
        "name": ["exact", "iexact", "in", "contains", "icontains", "isempty"],
    }


class RaceSerializer(ModelSerializer):
    class Meta:
        model = Race
        fields = "__all__"


class RaceViewSet(ModelViewSet):
    filename = "user_details_races"
    queryset = Race.objects.all()
    serializer_class = RaceSerializer
    filterset_fields = {
        "id": ["exact", "in", "isnull"],
        "name": ["exact", "iexact", "in", "contains", "icontains", "isempty"],
    }


class EthnicitySerializer(ModelSerializer):
    class Meta:
        model = Ethnicity
        fields = "__all__"


class EthnicityViewSet(ModelViewSet):
    filename = "user_details_ethnicities"
    queryset = Ethnicity.objects.all()
    serializer_class = EthnicitySerializer
    filterset_fields = {
        "id": ["exact", "in", "isnull"],
        "name": ["exact", "iexact", "in", "contains", "icontains", "isempty"],
    }


class EducationLevelSerializer(ModelSerializer):
    class Meta:
        model = EducationLevel
        fields = "__all__"


class EducationLevelViewSet(ModelViewSet):
    filename = "user_details_ethnicities"
    queryset = EducationLevel.objects.all()
    serializer_class = EducationLevelSerializer
    filterset_fields = {
        "id": ["exact", "in", "isnull"],
        "name": ["exact", "iexact", "in", "contains", "icontains", "isempty"],
    }


class UserDetailsSerializer(ModelSerializer):
    gender_name = CharField(source="gender.name", default=None, read_only=True)
    race_name = CharField(source="race.name", default=None, read_only=True)
    ethnicity_name = CharField(source="ethnicity.name", default=None, read_only=True)
    education_level_name = CharField(source="education_level.name", default=None, read_only=True)

    def get_fields(self):
        fields = super().get_fields()
        disable_fields, require_fields = UserDetailsCustomization.disable_require_fields()
        for disable_field in disable_fields:
            if disable_field in fields:
                del fields[disable_field]
        for require_field in require_fields:
            fields[require_field].required = True
            fields[require_field].allow_blank = False
        return fields

    class Meta:
        model = UserDetails
        exclude = ("user",)


class UserWithDetailsSerializer(UserSerializer):
    details = UserDetailsSerializer(required=False)

    class Meta(UserSerializer.Meta):
        UserSerializer.Meta.expandable_fields.update(
            {
                "gender": ("NEMO_user_details.api.GenderSerializer", {"source": "details.gender"}),
                "race": ("NEMO_user_details.api.RaceSerializer", {"source": "details.race"}),
                "ethnicity": ("NEMO_user_details.api.EthnicitySerializer", {"source": "details.ethnicity"}),
                "education_level": ("NEMO_user_details.api.EducationLevel", {"source": "details.education_level"}),
            }
        )

    def get_fields(self):
        fields = super().get_fields()
        detail_fields = fields.pop("details", {})
        if detail_fields:
            for key, value in detail_fields.fields.items():
                if key != "id":
                    # reset the source to details
                    value.source = "details." + value.source
                    value.source_attrs = value.source.split(".")
                    fields[key] = value
        return fields

    def validate(self, attrs):
        attributes_data = dict(attrs)
        details_attrs = attributes_data.pop("details", {})
        super().validate(attributes_data)
        user_details = get_user_details(self.instance)
        for details_attr, details_value in details_attrs.items():
            setattr(user_details, details_attr, details_value)
        UserDetailsSerializer().full_clean(user_details, exclude=["user"])
        return attrs

    def update(self, instance, validated_data) -> User:
        data = deepcopy(validated_data)
        details_data = data.pop("details", {})
        user_instance = super().update(instance, data)
        user_details = get_user_details(user_instance)
        UserDetailsSerializer().update(user_details, details_data)
        return user_instance

    def create(self, validated_data) -> User:
        data = deepcopy(validated_data)
        details_data = data.pop("details", {})
        user_instance = super().create(data)
        details_data["user"] = user_instance
        UserDetailsSerializer().create(details_data)
        return user_instance


class UserWithDetailsViewSet(UserViewSet):
    serializer_class = UserWithDetailsSerializer



import importlib
from datetime import timedelta
from functools import wraps
from http import HTTPStatus
from logging import getLogger
from typing import List, Optional
from urllib.parse import urljoin

import requests
from NEMO.decorators import any_staff_required
from NEMO.forms import UserForm
from NEMO.models import Area, AreaAccessRecord, PhysicalAccessLevel, Project, Qualification, Tool, User
from NEMO.utilities import render_combine_responses
from NEMO.views.customization import ApplicationCustomization, UserCustomization
from NEMO.views.users import create_or_modify_user, get_identity_service, readonly_users
from django.apps import apps
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.db.models import Max
from django.http.response import HttpResponseRedirectBase
from django.shortcuts import render
from django.utils import timezone
from django.views.decorators.http import require_http_methods

from NEMO_user_details.forms import UserDetailsForm
from NEMO_user_details.models import get_user_details

users_logger = getLogger(__name__)


# TODO: Remove when NEMO 5.3.0 is released
def replace_function(old_function_name, raise_exception=True):
    try:
        pkg, fun_name = old_function_name.rsplit(".", 1)
        pkg_mod = importlib.import_module(pkg)
        old_function = getattr(pkg_mod, fun_name)
    except:
        old_function = None
        if raise_exception:
            raise
        else:
            users_logger.warning(f"Could not replace function: {old_function_name}", exc_info=True)

    def decorator(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            return function(old_function, *args, **kwargs)

        if old_function:
            setattr(pkg_mod, fun_name, wrapper)
        return wrapper

    return decorator


# This is a little tricky since we don't want to rewrite all the logic (especially identity service stuff).
# On GET, we are using our user details template and inserting it onto the original page
# On POST, we have 3 cases:
#   1. original form and details form have no errors, so we proceed and save our details after the user is saved
#   2. original form has errors, in which case we insert our form into the original page
#   3. original form has no errors but details form does, so we have to re-render without calling the original method
@any_staff_required
@require_http_methods(["GET", "POST"])
def create_or_modify_user_and_details(request, user_id):
    try:
        user = User.objects.get(id=user_id)
    except (User.DoesNotExist, ValueError):
        user = None
    user_details = get_user_details(user)

    details_dictionary = get_details_dictionary(user)
    readonly = readonly_users(request)
    if readonly or request.method == "GET":
        details_dictionary["form_details"] = UserDetailsForm(instance=user_details)
        original_response = create_or_modify_user(request, user_id)
        return render_combine_responses(
            request, original_response, "NEMO_user_details/user_details.html", details_dictionary
        )
    elif request.method == "POST":
        form = UserForm(request.POST, instance=user)
        details_form = UserDetailsForm(request.POST, instance=user_details)

        details_dictionary["form_details"] = details_form
        if form.is_valid() and details_form.is_valid():
            original_response = create_or_modify_user(request, user_id)
            if isinstance(original_response, HttpResponseRedirectBase):
                # If it's a redirect, that means everything went ok, we can save the corresponding user details
                details_form.instance.user = User.objects.get(username=form.cleaned_data["username"])
                details_form.save()
                return original_response
            else:
                # There was an error and user was not saved, render original combined with details
                return render_combine_responses(
                    request, original_response, "NEMO_user_details/user_details.html", details_dictionary
                )
        elif not form.is_valid():
            # Original form isn't valid, everything we need is in the original response
            original_response = create_or_modify_user(request, user_id)
            return render_combine_responses(
                request, original_response, "NEMO_user_details/user_details.html", details_dictionary
            )
        else:
            # This case is a bit messy, but it's extremely unlikely to happen
            # Original form is valid, but ours isn't. We cannot proceed with original response.
            # We need to rebuild the original dictionary and send it back.
            dictionary = get_original_dictionary_for_errors(request, user)
            dictionary["form"] = form
            response = render(request, "users/create_or_modify_user.html", dictionary)
            return render_combine_responses(
                request, response, "NEMO_user_details/user_details.html", details_dictionary
            )


def get_details_dictionary(user: Optional[User]):
    return {
        "groups": Group.objects.all(),
        "user_groups": Group.objects.filter(user=user.id) if user else [],
    }


def get_original_dictionary_for_errors(request, user):
    # Original dictionary that would be returned in case of error. See NEMO.users.create_or_modify_user method
    # This is copy-pasted directly from the create_or_modify_user method
    identity_service = get_identity_service()
    # Get access levels and sort by area category
    access_levels = list(PhysicalAccessLevel.objects.all().only("name", "area"))
    access_level_for_sort = list(
        set([ancestor for access in access_levels for ancestor in access.area.get_ancestors(include_self=True)])
    )
    access_level_for_sort.sort(key=lambda x: x.tree_category())
    area_access_levels = Area.objects.filter(id__in=[area.id for area in access_level_for_sort])
    dict_area = {}
    for access in access_levels:
        dict_area.setdefault(access.area.id, []).append(access)

    readonly = readonly_users(request)
    dictionary = {
        "projects": Project.objects.filter(active=True, account__active=True),
        "tools": Tool.objects.filter(visible=True),
        "qualifications": Qualification.objects.filter(user=user),
        "area_access_dict": dict_area,
        "area_access_levels": area_access_levels,
        "one_year_from_now": timezone.localdate() + timedelta(days=365),
        "identity_service_available": identity_service.get("available", False),
        "identity_service_domains": identity_service.get("domains", []),
        "allow_document_upload": UserCustomization.get_bool("user_allow_document_upload"),
        "readonly": readonly,
    }

    last_access = AreaAccessRecord.objects.filter(customer=user).values("area_id").annotate(max_date=Max("start"))
    dictionary["last_access"] = {item["area_id"]: item["max_date"] for item in last_access}

    try:
        # Try to load qualification levels if NEMO_ce is installed
        from NEMO.models import QualificationLevel

        dictionary["qualification_levels"]: QualificationLevel.objects.all()
    except:
        pass

    timeout = identity_service.get("timeout", 3)
    site_title = ApplicationCustomization.get("site_title")
    if dictionary["identity_service_available"]:
        try:
            result = requests.get(urljoin(identity_service["url"], "/areas/"), timeout=timeout)
            if result.status_code == HTTPStatus.OK:
                dictionary["externally_managed_physical_access_levels"] = result.json()
            else:
                dictionary["identity_service_available"] = False
                warning_message = f"The identity service encountered a problem while attempting to return a list of externally managed areas. The administrator has been notified to resolve the problem."
                dictionary["warning"] = warning_message
                warning_message += " The HTTP error was {}: {}".format(result.status_code, result.text)
                users_logger.error(warning_message)
        except Exception as e:
            dictionary["identity_service_available"] = False
            warning_message = f"There was a problem communicating with the identity service. {site_title} is unable to retrieve the list of externally managed areas. The administrator has been notified to resolve the problem."
            dictionary["warning"] = warning_message
            warning_message += " An exception was encountered: " + type(e).__name__ + " - " + str(e)
            users_logger.error(warning_message)
    elif identity_service:
        # display warning if identity service is defined but disabled
        dictionary["warning"] = (
            "The identity service is disabled. You will not be able to modify externally managed physical access levels, reset account passwords, or unlock accounts."
        )

    return dictionary


@replace_function("NEMO.views.history.get_log_entries", raise_exception=False)
def new_get_log_entries(old_function, item, content_type):
    logentries = list(old_function(item, content_type))
    if apps.is_installed("auditlog") and isinstance(item, User):
        from auditlog.models import LogEntry

        user_details = get_user_details(item)
        detail_logentries: List[LogEntry] = LogEntry.objects.filter(
            content_type=ContentType.objects.get_for_model(user_details), object_id=user_details.id
        )
        logentries.extend(detail_logentries)
    return logentries



from typing import Set

from NEMO.decorators import customization
from NEMO.views.customization import CustomizationBase


@customization(title="User details", key="user_details")
class UserDetailsCustomization(CustomizationBase):
    variables = {
        "user_details_enable_groups": "",
        "user_details_enable_emergency_contact": "",
        "user_details_enable_phone_number": "",
        "user_details_enable_race": "",
        "user_details_enable_gender": "",
        "user_details_enable_ethnicity": "",
        "user_details_enable_education_level": "",
        "user_details_enable_employee_id": "",
        "user_details_enable_orcid": "",
        "user_details_enable_scopus_id": "",
        "user_details_enable_researcher_id": "",
        "user_details_enable_google_scholar_id": "",
        "user_details_require_groups": "",
        "user_details_require_emergency_contact": "",
        "user_details_require_phone_number": "",
        "user_details_require_race": "",
        "user_details_require_gender": "",
        "user_details_require_ethnicity": "",
        "user_details_require_education_level": "",
        "user_details_require_employee_id": "",
        "user_details_require_orcid": "",
        "user_details_require_scopus_id": "",
        "user_details_require_researcher_id": "",
        "user_details_require_google_scholar_id": "",
    }

    @classmethod
    def disable_require_fields(cls) -> (Set[str], Set[str]):
        # This function returns 2 lists:
        # - the first one containing fields to disable
        # - the second one containing fields to require
        # Here we rely on the order of variables where require field option is set after enable
        disable_fields, require_fields = set(), set()
        for var in cls.variables:
            if var.startswith("user_details_enable_"):
                enable_field_name = var.replace("user_details_enable_", "")
                if not cls.get_bool(var):
                    disable_fields.add(enable_field_name)
            elif var.startswith("user_details_require_"):
                require_field_name = var.replace("user_details_require_", "")
                if cls.get_bool(var):
                    require_fields.add(require_field_name)
                    if require_field_name in disable_fields:
                        disable_fields.remove(require_field_name)
        return disable_fields, require_fields



from django.contrib.auth.models import Group
from django.forms import HiddenInput, ModelMultipleChoiceField, models

from NEMO_user_details.customizations import UserDetailsCustomization
from NEMO_user_details.models import UserDetails


# Base details form for admin. We are setting disabled and required attributes here
class UserDetailsAdminForm(models.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        disable_fields, require_fields = UserDetailsCustomization.disable_require_fields()
        for field_name in require_fields:
            if field_name in self.fields:
                self.fields[field_name].required = True
        for field_name in disable_fields:
            if field_name in self.fields:
                self.fields[field_name].disabled = True
                self.fields[field_name].required = False
                self.fields[field_name].widget = HiddenInput()

    class Meta:
        model = UserDetails
        exclude = ["user"]


# Details form for users page (groups are already on the admin form for users, but we need to add them explicitly here)
class UserDetailsForm(UserDetailsAdminForm):
    groups = ModelMultipleChoiceField(
        queryset=Group.objects.all(),
        required=False,
    )

    def save(self, commit=True):
        details: UserDetails = super().save(commit=commit)
        if UserDetailsCustomization.get_bool("user_details_enable_groups"):
            details.user.groups.set(self.cleaned_data["groups"])
        return details



from NEMO.urls import router, sort_urls
from django.urls import path, re_path

from NEMO_user_details import api
from NEMO_user_details.views import email, users


def replace_api_url(url_to_replace, new_config):
    for reg in router.registry:
        if reg[0] == url_to_replace:
            router.registry.remove(reg)
    router.register(*new_config)


router.register(r"users_details/education_level", api.EducationLevelViewSet)
router.register(r"users_details/ethnicity", api.EthnicityViewSet)
router.register(r"users_details/gender", api.GenderViewSet)
router.register(r"users_details/race", api.RaceViewSet)
router.register(r"users_details/users", api.UserWithDetailsViewSet, basename="users_with_details")
replace_api_url("users", (r"users", api.UserWithDetailsViewSet))
router.registry.sort(key=sort_urls)

urlpatterns = [
    # Override modify user page to add user details fields
    re_path(
        r"^user/(?P<user_id>\d+|new)/",
        users.create_or_modify_user_and_details,
        name="create_or_modify_user_and_details",
    ),
    # Override broadcast email to add new search by groups
    path("email_broadcast/", email.email_broadcast, name="email_broadcast"),
    re_path(
        r"^email_broadcast/(?P<audience>tool|area|account|project|project-pis|user|tool-reservation|group)/$",
        email.email_broadcast,
        name="email_broadcast",
    ),
]



# Generated by Django 3.2.16 on 2022-12-26 02:50

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("NEMO_user_details", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="userdetails",
            name="rate_category_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="userdetails",
            name="rate_category_type",
            field=models.ForeignKey(
                blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to="contenttypes.contenttype"
            ),
        ),
    ]



from NEMO.models import BaseModel, User
from NEMO.views.constants import CHAR_FIELD_MEDIUM_LENGTH
from django.core.exceptions import ValidationError
from django.db import models


class Ethnicity(BaseModel):
    name = models.CharField(max_length=CHAR_FIELD_MEDIUM_LENGTH, unique=True, help_text="The name of the ethnicity")

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "Ethnicities"
        ordering = ["name"]


class Race(BaseModel):
    name = models.CharField(max_length=CHAR_FIELD_MEDIUM_LENGTH, unique=True, help_text="The name of the race")

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class Gender(BaseModel):
    name = models.CharField(max_length=CHAR_FIELD_MEDIUM_LENGTH, unique=True, help_text="The name of the gender")

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class EducationLevel(BaseModel):
    name = models.CharField(
        max_length=CHAR_FIELD_MEDIUM_LENGTH, unique=True, help_text="The name of the education level"
    )

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class UserDetails(BaseModel):
    user = models.OneToOneField(User, related_name="details", on_delete=models.CASCADE)
    gender = models.ForeignKey(Gender, null=True, blank=True, help_text="The user's gender", on_delete=models.SET_NULL)
    race = models.ForeignKey(Race, null=True, blank=True, help_text="The user's race", on_delete=models.SET_NULL)
    ethnicity = models.ForeignKey(
        Ethnicity, null=True, blank=True, help_text="The user's ethnicity", on_delete=models.SET_NULL
    )
    education_level = models.ForeignKey(
        EducationLevel, null=True, blank=True, help_text="The user's education level", on_delete=models.SET_NULL
    )
    emergency_contact = models.CharField(
        max_length=CHAR_FIELD_MEDIUM_LENGTH, blank=True, help_text="The user's emergency contact information"
    )
    phone_number = models.CharField(max_length=40, blank=True, help_text="The user's phone number")
    employee_id = models.CharField(
        max_length=CHAR_FIELD_MEDIUM_LENGTH, null=True, blank=True, help_text="The user's internal employee id"
    )
    orcid = models.CharField(
        verbose_name="ORCID", max_length=CHAR_FIELD_MEDIUM_LENGTH, null=True, blank=True, help_text="The user's ORCID"
    )
    scopus_id = models.CharField(
        verbose_name="Scopus ID",
        max_length=CHAR_FIELD_MEDIUM_LENGTH,
        null=True,
        blank=True,
        help_text="The user's Scopus ID",
    )
    researcher_id = models.CharField(
        verbose_name="Researcher ID",
        max_length=CHAR_FIELD_MEDIUM_LENGTH,
        null=True,
        blank=True,
        help_text="The user's Web of Science Researcher ID",
    )
    google_scholar_id = models.CharField(
        verbose_name="Google scholar ID",
        max_length=CHAR_FIELD_MEDIUM_LENGTH,
        null=True,
        blank=True,
        help_text="The user's Google Scholar ID",
    )

    def unique_identifier_field_names(self):
        return ["employee_id", "orcid", "scopus_id", "researcher_id", "google_scholar_id"]

    def clean(self):
        for field_name in self.unique_identifier_field_names():
            field_value = getattr(self, field_name)
            if field_value:
                not_unique = UserDetails.objects.filter(**{field_name: field_value}).exclude(id=self.id)
                if not_unique.exists():
                    raise ValidationError(
                        {field_name: f"A user with this {self._meta.get_field(field_name).name} already exists."}
                    )

    def __str__(self):
        return f"{self.user.get_name()}'s details"


def get_user_details(user: User):
    try:
        user_details = user.details
    except (UserDetails.DoesNotExist, AttributeError):
        user_details = UserDetails(user=user)
    return user_details



from django.apps import AppConfig


class NEMOUserDetailsConfig(AppConfig):
    name = "NEMO_user_details"
    verbose_name = "NEMO User details"
    default_auto_field = "django.db.models.AutoField"

    def ready(self):
        # Add group as a valid audience
        from NEMO.forms import EmailBroadcastForm

        EmailBroadcastForm.base_fields["audience"].choices.append(("group", "group"))



# Generated by Django 3.2.19 on 2023-05-25 18:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_details", "0003_version_1_3_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="userdetails",
            name="employee_id",
            field=models.CharField(blank=True, help_text="The user's internal employee id", max_length=255),
        ),
    ]



# Generated by Django 3.2.22 on 2023-12-13 15:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_details", "0005_alter_userdetails_employee_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="userdetails",
            name="orcid",
            field=models.CharField(
                blank=True, help_text="The user's ORCID", max_length=255, null=True, verbose_name="ORCID"
            ),
        ),
    ]



# Generated by Django 3.2.16 on 2022-12-24 02:54

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Ethnicity",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(help_text="The name of the ethnicity", max_length=200, unique=True)),
            ],
            options={
                "verbose_name_plural": "Ethnicities",
                "ordering": ["name"],
            },
        ),
        migrations.CreateModel(
            name="Gender",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(help_text="The name of the gender", max_length=200, unique=True)),
            ],
            options={
                "ordering": ["name"],
            },
        ),
        migrations.CreateModel(
            name="Race",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(help_text="The name of the race", max_length=200, unique=True)),
            ],
            options={
                "ordering": ["name"],
            },
        ),
        migrations.CreateModel(
            name="UserDetails",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "emergency_contact",
                    models.CharField(blank=True, max_length=250, help_text="The user's emergency contact information"),
                ),
                ("phone_number", models.CharField(blank=True, max_length=40, help_text="The user's phone number")),
                (
                    "ethnicity",
                    models.ForeignKey(
                        blank=True,
                        help_text="The user's ethnicity",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="NEMO_user_details.ethnicity",
                    ),
                ),
                (
                    "gender",
                    models.ForeignKey(
                        blank=True,
                        help_text="The user's gender",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="NEMO_user_details.gender",
                    ),
                ),
                (
                    "race",
                    models.ForeignKey(
                        blank=True,
                        help_text="The user's race",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="NEMO_user_details.race",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE, related_name="details", to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
